from django.conf import settings
from rest_framework import serializers


class AbstractTranslatedSerializer(serializers.ModelSerializer):
    """
    Serializer for models with translated fields. Automatically handles translation
    fields based on settings.LANGUAGES.
    """

    def _translate_json(self, instance, field):
        """
        Generates a dictionary with translations for a given field.

        :param instance: The model instance.
        :param field: The base field name.
        :return: A dictionary of translations, e.g., {'en': value, 'uz': value}.
        """
        return {lang: getattr(instance, f"{field}_{lang}", None) for lang, _ in settings.LANGUAGES}

    def to_representation(self, instance):
        """
        Customizes the serialized representation of the instance.

        :param instance: The model instance.
        :return: A dictionary representation of the instance.
        """
        representation = super().to_representation(instance)

        translated_fields = getattr(self.Meta, "translated_fields", [])
        translation_mode = getattr(self.Meta, "translated", 0)  # Default to 0

        for field in translated_fields:
            if translation_mode == 1:
                representation[field] = self._translate_json(instance, field)
            elif translation_mode == 2:
                for lang, _ in settings.LANGUAGES:
                    field_name = f"{field}_{lang}"
                    representation[field_name] = getattr(instance, field_name, None)

        return representation

    def to_internal_value(self, data):
        """
        Converts incoming data to a validated dictionary suitable for the model.

        :param data: The input data.
        :return: A validated dictionary of field values.
        """
        internal_value = super().to_internal_value(data)

        translated_fields = getattr(self.Meta, "translated_fields", [])
        for field in translated_fields:
            for lang, _ in settings.LANGUAGES:
                translated_field = f"{field}_{lang}"
                if translated_field in data:
                    internal_value[translated_field] = data[translated_field]

        return internal_value



from rest_framework import permissions


class HasRole(permissions.BasePermission):
    """
    DRF Has role permission class
    example:
        class TestView(views.ListApiView):
            permission_classes = (HasRole(choices.RoleChoice.ADMIN),)
    """

    def __init__(self, roles: list) -> None:
        super().__init__()
        self.roles = roles

    def __call__(self, *args, **kwargs):
        return self

    def has_permission(self, request, view):
        return request.user.role in self.roles



"""
Base celery tasks
"""

from importlib import import_module

from celery import shared_task
import os
from django.utils.translation import gettext as _
import logging


@shared_task
def SendConfirm(phone, code):
    try:
        service = getattr(
            import_module(os.getenv("OTP_MODULE")), os.getenv("OTP_SERVICE")
        )()
        service.send_sms(
            phone, _("Sizning Tasdiqlash ko'dingiz: %(code)s") % {"code": code}
        )
        logging.info("Sms send: %s-%s" % (phone, code))
    except Exception as e:
        logging.error(
            "Error: {phone}-{code}\n\n{error}".format(phone=phone, code=code, error=e)
        )  # noqa
        raise Exception("Sms yuborishda xatolik yuzaga keldi")



import importlib

from django.conf import settings
from django.core import management


class Command(management.BaseCommand):
    help = "seeder database with"

    def print(self, message, is_type="success"):
        if is_type == "success":
            self.stdout.write(self.style.SUCCESS(message))
        else:
            self.stdout.write(self.style.ERROR(message))

    def handle(self, *args, **options):
        seeders = settings.SEEDERS if hasattr(settings, "SEEDERS") else []

        if len(seeders) == 0:
            self.print(
                "SEEDERS not defined:\n\nsettings file add SEEDERS variable",
                "error",
            )
            return

        for seeder in seeders:
            class_name = str(seeder).split(".")[-1]
            module_path = ".".join(str(seeder).split(".")[:-1])
            module = importlib.import_module(module_path)
            my_class = getattr(module, class_name)()

            if not hasattr(my_class, "run"):
                self.print("run function not found", "error")
                return
            try:
                my_class.run()
            except Exception as e:
                self.print(f"ERROR: {class_name} {e}", "error")
                return
            self.print(f"SUCCESS: {class_name}")



from django.utils.translation import gettext as _


class BaseViewSetMixin:
    def finalize_response(self, request, response, *args, **kwargs):
        if response.status_code >= 400:
            response.data = {
                "status": False,
                "data": response.data
            }
        else:
            response.data = {
                "status": True,
                "data": response.data,
            }

        return super().finalize_response(request, response, *args, **kwargs)



# Generated by Django 5.1.3 on 2024-12-20 14:37

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SmsConfirm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.IntegerField()),
                ('try_count', models.IntegerField(default=0)),
                ('resend_count', models.IntegerField(default=0)),
                ('phone', models.CharField(max_length=20)),
                ('expire_time', models.DateTimeField(blank=True, null=True)),
                ('unlock_time', models.DateTimeField(blank=True, null=True)),
                ('resend_unlock_time', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]


import math
from datetime import datetime, timedelta
from django.db import models

from .base import AbstractBaseModel


class SmsConfirm(AbstractBaseModel):
    SMS_EXPIRY_SECONDS = 120
    RESEND_BLOCK_MINUTES = 10
    TRY_BLOCK_MINUTES = 2
    RESEND_COUNT = 5
    TRY_COUNT = 10

    code = models.IntegerField()
    try_count = models.IntegerField(default=0)
    resend_count = models.IntegerField(default=0)
    phone = models.CharField(max_length=20)
    expire_time = models.DateTimeField(null=True, blank=True)
    unlock_time = models.DateTimeField(null=True, blank=True)
    resend_unlock_time = models.DateTimeField(null=True, blank=True)

    def sync_limits(self):
        if self.resend_count >= self.RESEND_COUNT:
            self.try_count = 0
            self.resend_count = 0
            self.resend_unlock_time = datetime.now() + timedelta(minutes=self.RESEND_BLOCK_MINUTES)
        elif self.try_count >= self.TRY_COUNT:
            self.try_count = 0
            self.unlock_time = datetime.now() + timedelta(minutes=self.TRY_BLOCK_MINUTES)

        if self.resend_unlock_time is not None and self.resend_unlock_time.timestamp() < datetime.now().timestamp():
            self.resend_unlock_time = None

        if self.unlock_time is not None and self.unlock_time.timestamp() < datetime.now().timestamp():
            self.unlock_time = None
        self.save()

    def is_expired(self):
        return (
            self.expire_time.timestamp() < datetime.now().timestamp()
            if hasattr(self.expire_time, "timestamp")
            else None
        )

    def is_block(self):
        return self.unlock_time is not None

    def reset_limits(self):
        self.try_count = 0
        self.resend_count = 0
        self.unlock_time = None

    def interval(self, time):
        expire = time.timestamp() - datetime.now().timestamp()
        minutes = math.floor(expire / 60)
        expire -= minutes * 60
        expire = math.floor(expire)

        return f"{minutes:02d}:{expire:02d}"

    def __str__(self) -> str:
        return f"{self.phone} | {self.code}"
